import java.awt.*;
import java.util.*;
import javax.swing.*;

public class ChessBoard extends Board {

  class Selection {
    public int x, y;
  
    public Selection(int x, int y) {
      this.x = x;
      this.y = y;
    }
    
  }
 
  private int FIELD_SIZE = 30;
  private int X_OFFSET = 30;
  private int Y_OFFSET = 20;
  
  private GameClient owner;
  private String stateStr;
  private char state[][];
  private Vector selection;
  private char who;

  private boolean hasABoardState;
  
  private ImageIcon kingB, kingW, queenB, queenW, knightB, knightW,
                    bishopB, bishopW, pawnB, pawnW, rookB, rookW, splash;

  public ChessBoard(GameClient owner) {
    this.owner = owner;
    stateStr = "";
    state = new char[8][8];
    selection = new Vector();
    initComponents();
    kingB = new ImageIcon("chess/kingb.gif");
    kingW = new ImageIcon("chess/kingw.gif");
    queenB = new ImageIcon("chess/queenb.gif");
    queenW = new ImageIcon("chess/queenw.gif");
    knightB = new ImageIcon("chess/knightb.gif");
    knightW = new ImageIcon("chess/knightw.gif");
    rookB = new ImageIcon("chess/rookb.gif");
    rookW = new ImageIcon("chess/rookw.gif");
    bishopB = new ImageIcon("chess/bishopb.gif");
    bishopW = new ImageIcon("chess/bishopw.gif");
    pawnB = new ImageIcon("chess/pawnb.gif");
    pawnW = new ImageIcon("chess/pawnw.gif");
    splash = new ImageIcon("chess/splash.gif");
    who = '_';
    hasABoardState = false;
  }

  private void initComponents() {//GEN-BEGIN:initComponents
    addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(java.awt.event.MouseEvent evt) {
        mouseClicker(evt);
      }
    });
  }

  private void mouseClicker(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseClicker
    boolean finished = false;
    int x = evt.getX() - X_OFFSET;
    int y =  evt.getY() - Y_OFFSET;
    if ((x < 0) || (y < 0) || (x > FIELD_SIZE * 8) || (y > FIELD_SIZE * 8))
      return;
    x /= FIELD_SIZE;
    y /= FIELD_SIZE;
    if (selection == null)
      selection = new Vector();
    for (int i = 0; i < selection.size(); i++) {
      Selection s = (Selection)selection.elementAt(i);
      if ((s.x == x) && (s.y == y)) {
        if (i == selection.size() - 1) {
          selection.removeElementAt(selection.size() - 1);
          finished = true;
          break;
        }
        else
          return;
      }
    }
    if ((!finished) && (selection.size() < 2))
      selection.addElement(new Selection(x, y));
    this.repaint(X_OFFSET + x * FIELD_SIZE - 1, Y_OFFSET + y * FIELD_SIZE - 1,
                 FIELD_SIZE + 2, FIELD_SIZE + 2);
    owner.setMove(selectionToString());
  }

  private String selectionToString() {
    String result = "";
    if (selection != null)
      for (int i = 0; i < selection.size(); i++) {
        Selection s = (Selection)selection.elementAt(i);
        result += "" + (char)('a' + (char)s.x) + "" + (char)('1' + (char)(7 - s.y));
      }
    return result;
  }
    
  public void drawEmptyBoard(Graphics g) {
    int x, y, width;
    char c;
    Color white = new Color(240, 240, 200);
    Color black = new Color(120, 120, 100);
    FontMetrics fm = g.getFontMetrics();
    g.setColor(Color.white);
    for (x = 0; x < 8; x++) {
      c = (char)('A' + x);
      width = fm.charWidth(c);
      y = X_OFFSET + x * FIELD_SIZE + FIELD_SIZE/2 - width/2;
      g.drawString("" + c, y, Y_OFFSET - 6);
    }
    for (y = 0; y < 8; y++) {
      c = (char)('8' - y);
      width = fm.charWidth(c);
      x = X_OFFSET - width - 6;
      g.drawString("" + c, x, Y_OFFSET + (y + 1) * FIELD_SIZE - 9);
    }
    for (x = 0; x < 8; x++)
      for (y = 0; y < 8; y++) {
        if ((x + y) % 2 == 0)
          g.setColor(white);
        else
          g.setColor(black);
        g.fillRect(X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE,
                   FIELD_SIZE, FIELD_SIZE);
      }
    if (who == 'w')
      g.drawString("You have the white pieces.", X_OFFSET, 280);
    else if (who == 'b')
      g.drawString("You have the black pieces.", X_OFFSET, 280);
  }
    
  public void paint(Graphics g) {
    if (!hasABoardState) {
      splash.paintIcon(this, g, 0, 0);
      return;
    }
    int x, y, cy;
    Color lineColor, fillColor;
    drawEmptyBoard(g);
    if (selection != null)
      for (x = 0; x < selection.size(); x++) {
        Selection s = (Selection)selection.elementAt(x);
        switch (x) {
          case 0:  g.setColor(new Color(240, 40, 40)); break;
          default: g.setColor(new Color(40, 40, 240)); break;
        }
        g.fillRect(X_OFFSET + s.x * FIELD_SIZE, Y_OFFSET + s.y * FIELD_SIZE,
                   FIELD_SIZE, FIELD_SIZE);
      }
    lineColor = fillColor = null;
    if ((stateStr != null) && (stateStr.length() > 0)) {
      for (x = 0; x < 8; x++)
        for (y = 0; y < 8; y++) {
          ImageIcon i = null;
          switch (state[x][7 - y]) {
            case 'C': i = kingW; break;
            case 'c': i = kingB; break;
            case 'Q': i = queenW; break;
            case 'q': i = queenB; break;
            case 'R': i = rookW; break;
            case 'r': i = rookB; break;
            case 'B': i = bishopW; break;
            case 'b': i = bishopB; break;
            case 'K': i = knightW; break;
            case 'k': i = knightB; break;
            case 'P': i = pawnW; break;
            case 'p': i = pawnB; break;
          }
          if (i != null)
            i.paintIcon(this, g, X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE);
        }
    }
  }
  
  public void setBoardState(String aState) {
    hasABoardState = true;
    stateStr = aState;
    if ((stateStr != null) && (stateStr.length() == 64)) {
      for (int x = 0; x < 8; x++)
        for (int y = 0; y < 8; y++)
          state[x][y] = stateStr.charAt(x + 8 * y);
    }
    selection = new Vector();
    repaint();
  }

  public void setTimeOut(long ms) {
  }

  public void setWho(char who) {
    this.who = who;
  }

}
