/********************************************************************************
 * Player.java
 * Copyright (c) 2002 by Stefan Buettcher
 *
 * This class manages all players (users) that are connected to the Master.
 *
 *******************************************************************************/

import java.net.*;
import java.io.*;
import java.util.*;

public class Player implements Comparable {
  protected String name;       // the player's name
  public Match match;          // reference to the match he/she is currently in
  protected boolean canStart;  // has he/she set his/her canStart flag?
  protected Connection connection;

  public Player() {
    connection = null;
    canStart = false;
    match = null;
  }

  public Player(String aName) {
  // just an ordinary constructor
    name = aName;
    match = null;
    canStart = false;
    connection = null;
  }

  public void setCanStart(boolean can) {
    canStart = can;
  }

  public boolean getCanStart() {
    return canStart;
  }

  public void setMatch(Match aMatch) {
    match = aMatch;
  }

  public Match getMatch() {
    return match;
  }

  public void setConnection(Connection conn) {
    connection = conn;
  }
  
  public void sendMessage(String message) {
    if (connection != null)
      connection.sendMessage(message);
System.out.println("To " + name + ": " + message);
  }
  
  public String getName() {
    return name;
  }

  public String toString() {
    return name;
  }

  public int compareTo(Object o) {
  // Player instances must be comparable if we want to organize them in a tree
    return name.compareTo(o.toString());
  }

}
